@@ -1,46 +1,15 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  * vim: set ts=8 sw=4 et tw=99 ft=cpp:
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla SpiderMonkey JavaScript 1.9 code, released
- * June 12, 2009.
- *
- * The Initial Developer of the Original Code is
- *   the Mozilla Corporation.
- *
- * Contributor(s):
- *   Luke Wagner <lw@mozilla.com>
- *   Nicholas Nethercote <nnethercote@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef jsvector_h_
 #define jsvector_h_
 
+#include "mozilla/Attributes.h"
+
 #include "TemplateLib.h"
 #include "Utility.h"
 
@@ -52,7 +21,11 @@
 
 namespace js {
 
-template <class T, size_t N, class AllocPolicy>
+class TempAllocPolicy;
+
+template <class T,
+          size_t MinInlineCapacity = 0,
+          class AllocPolicy = TempAllocPolicy>
 class Vector;
 
 /*
@@ -207,6 +180,8 @@ struct VectorImpl<T, N, AP, true>
 template <class T, size_t N, class AllocPolicy>
 class Vector : private AllocPolicy
 {
+    // typedef typename tl::StaticAssert<!tl::IsPostBarrieredType<T>::result>::result _;
+
     /* utilities */
 
     static const bool sElemIsPod = tl::IsPodType<T>::result;
@@ -267,20 +242,24 @@ class Vector : private AllocPolicy
     size_t mReserved;   /* Max elements of reserved or used space in this vector. */
 #endif
 
-    AlignedStorage<sInlineBytes> storage;
+    mozilla::AlignedStorage<sInlineBytes> storage;
 
 #ifdef DEBUG
     friend class ReentrancyGuard;
     bool entered;
 #endif
 
-    Vector(const Vector &);
-    Vector &operator=(const Vector &);
+    Vector(const Vector &) MOZ_DELETE;
+    Vector &operator=(const Vector &) MOZ_DELETE;
 
     /* private accessors */
 
     bool usingInlineStorage() const {
-        return mBegin == (T *)storage.addr();
+        return mBegin == inlineStorage();
+    }
+
+    T *inlineStorage() const {
+        return (T *)storage.addr();
     }
 
     T *beginNoCheck() const {
@@ -343,7 +322,12 @@ class Vector : private AllocPolicy
         return mCapacity;
     }
 
-    T *begin() const {
+    T *begin() {
+        JS_ASSERT(!entered);
+        return mBegin;
+    }
+
+    const T *begin() const {
         JS_ASSERT(!entered);
         return mBegin;
     }
@@ -378,6 +362,23 @@ class Vector : private AllocPolicy
         return *(end() - 1);
     }
 
+    class Range {
+        friend class Vector;
+        T *cur, *end;
+        Range(T *cur, T *end) : cur(cur), end(end) {}
+      public:
+        Range() {}
+        bool empty() const { return cur == end; }
+        size_t remain() const { return end - cur; }
+        T &front() const { return *cur; }
+        void popFront() { JS_ASSERT(!empty()); ++cur; }
+        T popCopyFront() { JS_ASSERT(!empty()); return *cur++; }
+    };
+
+    Range all() {
+        return Range(begin(), end());
+    }
+
     /* mutators */
 
     /* If reserve(length() + N) succeeds, the N next appends are guaranteed to succeed. */
@@ -430,7 +431,7 @@ class Vector : private AllocPolicy
         internalAppendN(t, n);
     }
     template <class U> void infallibleAppend(const U *begin, const U *end) {
-        internalAppend(begin, PointerRangeSize(begin, end));
+        internalAppend(begin, mozilla::PointerRangeSize(begin, end));
     }
     template <class U> void infallibleAppend(const U *begin, size_t length) {
         internalAppend(begin, length);
@@ -471,6 +472,19 @@ class Vector : private AllocPolicy
      * shifting existing elements from |t + 1| onward one position lower.
      */
     void erase(T *t);
+
+    /*
+     * Measure the size of the Vector's heap-allocated storage.
+     */
+    size_t sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf) const;
+
+    /*
+     * Like sizeOfExcludingThis, but also measures the size of the Vector
+     * object (which must be heap-allocated) itself.
+     */
+    size_t sizeOfIncludingThis(JSMallocSizeOfFun mallocSizeOf) const;
+
+    void swap(Vector &other);
 };
 
 /* This does the re-entrancy check plus several other sanity checks. */
@@ -498,6 +512,9 @@ template <class T, size_t N, class AllocPolicy>
 JS_ALWAYS_INLINE
 Vector<T, N, AllocPolicy>::Vector(MoveRef<Vector> rhs)
     : AllocPolicy(rhs)
+#ifdef DEBUG
+    , entered(false)
+#endif
 {
     mLength = rhs->mLength;
     mCapacity = rhs->mCapacity;
@@ -555,7 +572,12 @@ Vector<T,N,AP>::~Vector()
  */
 template <class T, size_t N, class AP>
 STATIC_POSTCONDITION(!return || newCap >= curLength + lengthInc)
+#ifdef DEBUG
+/* gcc (ARM, x86) compiler bug workaround - See bug 694694 */
+JS_NEVER_INLINE bool
+#else
 inline bool
+#endif
 Vector<T,N,AP>::calculateNewCapacity(size_t curLength, size_t lengthInc,
                                      size_t &newCap)
 {
@@ -843,7 +865,7 @@ JS_ALWAYS_INLINE bool
 Vector<T,N,AP>::append(const U *insBegin, const U *insEnd)
 {
     REENTRANCY_GUARD_ET_AL;
-    size_t needed = PointerRangeSize(insBegin, insEnd);
+    size_t needed = mozilla::PointerRangeSize(insBegin, insEnd);
     if (mLength + needed > mCapacity && !growStorageBy(needed))
         return false;
 
@@ -968,6 +990,47 @@ Vector<T,N,AP>::replaceRawBuffer(T *p, size_t length)
 #endif
 }
 
+template <class T, size_t N, class AP>
+inline size_t
+Vector<T,N,AP>::sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf) const
+{
+    return usingInlineStorage() ? 0 : mallocSizeOf(beginNoCheck());
+}
+
+template <class T, size_t N, class AP>
+inline size_t
+Vector<T,N,AP>::sizeOfIncludingThis(JSMallocSizeOfFun mallocSizeOf) const
+{
+    return mallocSizeOf(this) + sizeOfExcludingThis(mallocSizeOf);
+}
+
+template <class T, size_t N, class AP>
+inline void
+Vector<T,N,AP>::swap(Vector &other)
+{
+    // TODO Implement N != 0
+    JS_STATIC_ASSERT(N == 0);
+
+    // This only works when inline storage is always empty.
+    if (!usingInlineStorage() && other.usingInlineStorage()) {
+        other.mBegin = mBegin;
+        mBegin = inlineStorage();
+    } else if (usingInlineStorage() && !other.usingInlineStorage()) {
+        mBegin = other.mBegin;
+        other.mBegin = other.inlineStorage();
+    } else if (!usingInlineStorage() && !other.usingInlineStorage()) {
+        Swap(mBegin, other.mBegin);
+    } else {
+        // This case is a no-op, since we'd set both to use their inline storage.
+    }
+
+    Swap(mLength, other.mLength);
+    Swap(mCapacity, other.mCapacity);
+#ifdef DEBUG
+    Swap(mReserved, other.mReserved);
+#endif
+}
+
 }  /* namespace js */
 
 #ifdef _MSC_VER
